@@ -1,21 +1,27 @@
 /*
-   $Id: label.cc,v 1.13 2008/05/28 22:04:13 ksterker Exp $
-
    (C) Copyright 2000/2001/2004 Joel Vennin
-   Part of the Adonthell Project http://adonthell.linuxgames.com
+   Part of the Adonthell Project <http://adonthell.nongnu.org>
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+   Adonthell is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-   See the COPYING file for more details
+   Adonthell is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Adonthell.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "label.h"
 
 u_int16 label::cursor_blink_cycle = 75; 
 
+#define LINE_LENGTH(line) (line.pos_x + line.offset_x/screen::scale())
+
 /**
    Constructor
 */
@@ -54,8 +60,7 @@ label::~label ()
 */
 void label::set_font (win_font & font)
 {
-    my_font_ = &font; 
-    //  build (true); 
+    my_font_ = &font;
 }
 
 
@@ -121,6 +126,10 @@ void label::resize (u_int16 l, u_int16 h)
 void label::set_form (const u_int8 form)
 {
     my_form_ = form;
+
+    if (my_text_.empty())
+    	return;
+
     build (true); 
 }
 
@@ -131,14 +140,14 @@ void label::set_form (const u_int8 form)
 void label::init_vec_cursor ()
 {
     // init the cursor
-    my_cursor_.pos_x = my_cursor_.pos_y = my_cursor_.line = my_cursor_.idx = 0;  
+	memset(&my_cursor_, 0, sizeof my_cursor_);
     
     // init the vector
     my_vect_.clear ();
     
     // create a line in the vector
     Sline_text tmp;
-    tmp.pos_x = tmp.idx_beg = tmp.idx_end = 0; 
+    memset(&tmp, 0, sizeof tmp);
     
     // add the new line at the beginning of the vector
     my_vect_.push_back (tmp);
@@ -156,8 +165,8 @@ void label::init_vec_cursor ()
 
 void label::build (const bool erase_all)
 {
-    if (my_font_ == NULL) return; 
-    set_mask (false);  
+    if (my_font_ == NULL) return;
+    set_mask (false);
     switch (my_form_)
     {
         case NOTHING :
@@ -176,7 +185,7 @@ void label::build (const bool erase_all)
             build_form_auto_size ();
             update_cursor ();
             draw_string (false);
-            break; 
+            break;
     }
     set_mask (true); 
 }
@@ -208,6 +217,7 @@ void label::build_form_nothing ()
 {
     // temporary variable
     u_int16 j, word_length, word_length_pix, start_idx;
+    s_int16 word_offset_x;
     
     // temporary line
     Sline_text line_tmp;
@@ -215,6 +225,7 @@ void label::build_form_nothing ()
     // we start at the beginning index of cursor line
     line_tmp.idx_beg = my_vect_[my_old_cursor_.line].idx_beg;  
     line_tmp.pos_x = 0;
+    line_tmp.offset_x = 0;
 
     // we start always at the begin index of the line
     start_idx = line_tmp.idx_beg;   
@@ -225,6 +236,7 @@ void label::build_form_nothing ()
     while (i != my_old_cursor_.line) { i++; ii++; } 
     my_vect_.erase (ii, my_vect_.end ());
 
+    u_int16 prev_c = 0;
     
     while (start_idx < my_text_.length () )
     { 
@@ -239,11 +251,14 @@ void label::build_form_nothing ()
             
             // init a Sline_text
             line_tmp.pos_x = 0; 
+            line_tmp.offset_x = 0;
             line_tmp.idx_beg = ++start_idx; 
+
+            prev_c = 0;
         }
         else if (my_text_[start_idx] == ' ')
         {
-            if ((*my_font_) [' '].length () + line_tmp.pos_x > length ())
+            if ((*my_font_) [' '].length () + LINE_LENGTH(line_tmp) > length ())
             {
                 line_tmp.idx_end = start_idx;
                 
@@ -252,22 +267,30 @@ void label::build_form_nothing ()
                 
                 // init a Sline_text
                 line_tmp.pos_x = 0;
+                line_tmp.offset_x = 0;
                 line_tmp.idx_beg = ++start_idx; 
                 
-            } else 
+                prev_c = 0;
+            }
+            else
             {
                 line_tmp.pos_x += (*my_font_) [' '].length ();
+                if (prev_c != 0) line_tmp.offset_x += my_font_->kerning(prev_c, ' ');
                 start_idx++;
+
+                prev_c = ' ';
             }
         }
         else
         { 
             // find a word
             
-            switch (find_word (start_idx, word_length, word_length_pix, line_tmp.pos_x))
+            switch (find_word (start_idx, word_length, word_length_pix, word_offset_x, prev_c, LINE_LENGTH(line_tmp)))
             {
                 case 0 : // enough place
                     line_tmp.pos_x += word_length_pix;
+                    line_tmp.offset_x += word_offset_x;
+
                     break;
                     
                 case 1 : // enough place,  but return at the next line 
@@ -275,18 +298,17 @@ void label::build_form_nothing ()
 
                     if (length () && height ())
                     {
-                        
-                        lock ();
+                        offset_x_ = line_tmp.offset_x;
                         fillrect (line_tmp.pos_x,
                                   (my_vect_.size () - start_line_) * my_font_->height (),
-                                  length () - line_tmp.pos_x,
+                                  length () - LINE_LENGTH(line_tmp),
                                   my_font_->height (), screen::trans_col () );  
-                        unlock (); 
                     }
                     line_tmp.idx_end = (start_idx - word_length) - 1;   
                     my_vect_.push_back (line_tmp); 
                     
                     line_tmp.pos_x = word_length_pix; 
+                    line_tmp.offset_x = word_offset_x;
                     line_tmp.idx_beg = start_idx - word_length;
                     
                     break;
@@ -303,10 +325,14 @@ void label::build_form_nothing ()
                             my_vect_.push_back (line_tmp);
                             
                             line_tmp.pos_x = 0;
+                            line_tmp.offset_x = 0;
                             line_tmp.idx_beg = j; 
                         }
                         line_tmp.pos_x += (*my_font_) [c].length (); 
+                        if (prev_c != 0) line_tmp.offset_x += my_font_->kerning(prev_c, c);
                         j++; 
+
+                        prev_c = c;
                     }
                     break;  
             } 
@@ -331,9 +357,7 @@ void label::build_form_auto_height ()
     if (new_size  != height ())
     {
         image tmp (length (), new_size);
-        tmp.lock (); 
         tmp.fillrect (0, 0, length (), new_size, screen::trans_col ()); 
-        tmp.unlock (); 
         draw (0, 0, 0, 0, length (), my_old_cursor_.pos_y + my_font_->height (), NULL, &tmp); 
         image::resize (length (), new_size); 
         tmp.draw (0, 0, NULL, this); 
@@ -352,28 +376,44 @@ void label::build_form_auto_size ()
     Sline_text line_tmp;
     
     line_tmp.pos_x = 0;
+    line_tmp.offset_x = 0;
     line_tmp.idx_beg = 0;
+    
     u_int16 i = 0, max_length = 0; 
+    u_int16 prev_c = 0;
     
     while ( i < my_text_.size ())
     {
         if (my_text_[i] == '\n')
         {
-            if (line_tmp.pos_x > max_length) max_length = line_tmp.pos_x; 
+            if (line_tmp.pos_x + LINE_LENGTH(line_tmp) > max_length)
+			{
+            	max_length = LINE_LENGTH(line_tmp);
+			}
             line_tmp.idx_end = i; 
             my_vect_.push_back (line_tmp);
             
             line_tmp.idx_beg = i+1;
             line_tmp.pos_x = 0; 
+            line_tmp.offset_x = 0;
+
+            prev_c = 0;
         }
         else
         {
-            line_tmp.pos_x += (*my_font_) [ucd (i)].length (); 
+        	u_int16 c = ucd (i);
+            line_tmp.pos_x += (*my_font_) [c].length ();
+            if (prev_c != 0) line_tmp.offset_x += my_font_->kerning(prev_c, c);
+            prev_c = c;
         }
         i++; 
     }
     
-    if (line_tmp.pos_x > max_length) max_length = line_tmp.pos_x;
+    if (LINE_LENGTH(line_tmp) > max_length)
+	{
+    	max_length = LINE_LENGTH(line_tmp);
+	}
+
     // the last line
     line_tmp.idx_end = i-1;
     my_vect_.push_back (line_tmp);
@@ -382,23 +422,40 @@ void label::build_form_auto_size ()
     image::resize (max_length, my_vect_.size () * my_font_->height ());  
 }
 
+void label::fit_text_width()
+{
+	u_int16 new_size = 0;
+	for (vector<Sline_text>::iterator i = my_vect_.begin(); i != my_vect_.end(); i++)
+	{
+	    if (LINE_LENGTH((*i)) > new_size)
+		{
+	    	new_size = LINE_LENGTH((*i));
+		}
+	}
+
+    if (new_size < length ())
+    {
+    	set_length(new_size);
+    }
+}
+
 void label::clean_surface (const bool erase_all)
 {
     if (length () && height ())
     {     
         if ( my_cursor_.idx != my_text_.length ())
         {
-            lock (); 
+        	offset_x_ = my_old_cursor_.offset_x;
             fillrect ( my_old_cursor_.pos_x, my_old_cursor_.pos_y, length () - my_old_cursor_.pos_x,
-                       my_font_->height (), screen::trans_col ()); 
+                       my_font_->height (), screen::trans_col (), NULL);
+            offset_x_ = 0;
             fillrect (0, my_old_cursor_.pos_y + my_font_->height (), length (),
                       height () -my_old_cursor_.pos_y + my_font_->height (), screen::trans_col ()); 
-            unlock ();  
-        } else if (erase_all) 
+        }
+        else if (erase_all)
         {
-            lock ();
+            offset_x_ = 0;
             fillrect (0, 0, length (), height (), screen::trans_col ()); 
-            unlock ();  
         }
     }
 }
@@ -414,22 +471,29 @@ void label::clean_surface (const bool erase_all)
 // length :
 
 // return 0 if enough size for this word,  1 if enough but must return on the next line, 2 if the word is bigger than the length 
-u_int8 label::find_word (u_int16 & index, u_int16 & wlength, u_int16 & wlengthpix, const u_int16 rlength)
+u_int8 label::find_word (u_int16 & index, u_int16 & wlength, u_int16 & wlengthpix, s_int16 & woffset, u_int16 & last_letter, const u_int16 rlength)
 {
+	u_int16 c = 0;
+
     wlength = index;
     wlengthpix = 0;
+    woffset = 0;
     while (index < my_text_.length ()  && my_text_[index] != ' ' && my_text_[index] != '\n')
     {
-        wlengthpix += (*my_font_) [ucd (index)].length (); 
+    	c = ucd (index);
+        wlengthpix += (*my_font_) [c].length ();
+        if (last_letter != 0) woffset += my_font_->kerning(last_letter, c);
+    	last_letter = c;
         index++;
     }
 
     // count of characters (which is != count of letters due to utf-8 encoding)
     wlength = index - wlength;
+	last_letter = c;
     
     // if size of word is bigger than the length of label 
-    if (wlengthpix < length () - rlength)  return 0;
-    else if (wlengthpix < length ())  return 1; 
+    if (wlengthpix + woffset/screen::scale() < length () - rlength)  return 0;
+    else if (wlengthpix + woffset/screen::scale() < length ())  return 1;
     return 2;
 }
 
@@ -461,20 +525,20 @@ void label::update_cursor ()
  
     // now find the x position of the cursor
     my_cursor_.pos_x = 0;
+    my_cursor_.offset_x = 0;
     
+    u_int16 prev_c = 0;
     u_int16 j = my_vect_[my_cursor_.line].idx_beg;
     while (j < my_cursor_.idx) {
-        my_cursor_.pos_x+= (*my_font_) [ucd (j)].length ();     
+    	u_int16 c = ucd (j);
+        my_cursor_.pos_x+= (*my_font_) [c].length ();
+        if (prev_c != 0) my_cursor_.offset_x += my_font_->kerning(prev_c, c);
+    	prev_c = c;
         j++;
     }
+
     // find y position
     my_cursor_.pos_y = (my_cursor_.line - start_line_) * my_font_->height (); 
-
-    if (my_cursor_.pos_y > height ())
-    {
-        
-
-    } 
 }
 
 
@@ -485,17 +549,19 @@ void label::draw_string (const bool at_cursor)
     u_int16 tmp_start_line;
     u_int16 tx = 0, ty = 0;
     u_int16 idx_cur_line, j; 
-    u_int16 c;
+    u_int16 c, prev_c = 0;
+    s_int16 ox = 0;
     
     // if not at cursor, we erase all
     clean_surface (!at_cursor); 
     
     if (at_cursor)
     { 
-        tmp_start_line =  my_old_cursor_.line; 
+        tmp_start_line = my_old_cursor_.line;
         tx = my_old_cursor_.pos_x;
+        ox = my_cursor_.offset_x; // my_cursor_ has been updated with the correct offset!
         idx_cur_line = my_old_cursor_.idx;
-        ty = (tmp_start_line - start_line_) * my_font_->height (); 
+        ty = (tmp_start_line - start_line_) * my_font_->height ();
     }
     else
     { 
@@ -511,9 +577,13 @@ void label::draw_string (const bool at_cursor)
         c = ucd (j);
         if (c != '\n' && my_font_->in_table (c))
         {
+            if (prev_c != 0) ox += my_font_->kerning(prev_c, c);
+        	offset_x_ = ox;
             (*my_font_) [c].draw (tx, ty, NULL, this);
             tx += (*my_font_) [c].length ();
+            prev_c = c;
         }
+        else prev_c = 0;
     }
     ty += my_font_->height ();
     tmp_start_line++; 
@@ -523,6 +593,7 @@ void label::draw_string (const bool at_cursor)
     while (tmp_start_line < my_vect_.size ())
     {
         tx = 0; 
+        ox = 0;
         for (j = my_vect_[tmp_start_line].idx_beg;
              j <  my_vect_[tmp_start_line].idx_end + 1 ;
              j++)
@@ -530,9 +601,13 @@ void label::draw_string (const bool at_cursor)
             c = ucd (j);
             if (my_font_->in_table (c))
             {
+                if (prev_c != 0) ox += my_font_->kerning(prev_c, c);
+            	offset_x_ = ox;
                 (*my_font_) [c].draw (tx, ty, NULL, this);
                 tx += (*my_font_) [c].length (); 
+                prev_c = c;
             }
+            else prev_c = 0;
         }
         ty += my_font_->height ();
         tmp_start_line++;
@@ -547,7 +622,9 @@ bool label::update ()
         if (! (height () && length ())) return true;  
         if (cursor_cur_blink_ == cursor_blink_cycle)
         {
-            cursor_draw (); 
+            set_mask(false);
+            cursor_draw ();
+            set_mask(true);
             cursor_cur_blink_ = 0; 
         }else if (cursor_cur_blink_ == (cursor_blink_cycle >> 1))
             cursor_undraw ();   
@@ -562,8 +639,9 @@ void label::cursor_draw ()
 {
      // draw the cursor
     u_int16 idx = my_cursor_.idx;
+	offset_x_ = my_cursor_.offset_x;
     if (last_letter (idx)  || my_text_[idx] == '\n')  
-        my_font_->cursor->draw (my_cursor_.pos_x, my_cursor_.pos_y,NULL, this);  
+        my_font_->cursor->draw (my_cursor_.pos_x, my_cursor_.pos_y,NULL, this);
     else
         my_font_->cursor->draw (my_cursor_.pos_x, my_cursor_.pos_y,0, 0, 
                                 (*my_font_) [ucd (idx)].length (),
@@ -574,14 +652,13 @@ void label::cursor_undraw ()
 { 
     // draw letter instead  
     u_int16 idx = my_cursor_.idx;
+	offset_x_ = my_cursor_.offset_x;
     if (last_letter (idx) || my_text_[idx] == '\n') 
     {
-        lock (); 
         fillrect(my_cursor_.pos_x, my_cursor_.pos_y,
                  my_font_->cursor->length () ,
                  my_font_->cursor->height(),
                  screen::trans_col());
-        unlock (); 
     }
     else (*my_font_) [ucd (idx)].draw (my_cursor_.pos_x, my_cursor_.pos_y, NULL, this);
 }
@@ -595,7 +672,6 @@ bool label::last_letter (u_int16 idx)
 
 bool label::input_update ()
 {
-
     if(input::has_been_pushed(KEY_CURSOR_NEXT)) 
     {
         if (! (height () && length ())) return false;  
@@ -618,6 +694,7 @@ void label::cursor_next ()
     if (!moveable_cursor_) return; 
     if (my_cursor_.idx < my_text_.length ()) 
     {
+    	// TODO: kerning
         u_int8 count;
         if (my_cursor_.idx < my_text_.length () - 2 && (u_int8) my_text_[my_cursor_.idx+1] == 0xEF) count = 3;
         else if (my_cursor_.idx < my_text_.length () - 1 && (u_int8) my_text_[my_cursor_.idx+1] == 0xC3) count = 2;
@@ -634,6 +711,7 @@ void label::cursor_previous ()
     if (!moveable_cursor_) return; 
     if (my_cursor_.idx > 0) 
     {
+    	// TODO: kerning
         u_int8 count;
         if (my_cursor_.idx > 2 && (u_int8) my_text_[my_cursor_.idx-3] == 0xEF) count = 3;
         else if (my_cursor_.idx > 1 && (u_int8) my_text_[my_cursor_.idx-2] == 0xC3) count = 2;
@@ -645,7 +723,7 @@ void label::cursor_previous ()
 }
 
 
-const string label::text_string () const
+const string & label::text_string () const
 {
     return my_text_;  
 }
@@ -656,22 +734,21 @@ const char * label::text_char () const
 }
 
 // utf-8 --> utf-16
-u_int16 label::ucd (u_int16 & idx)
+u_int16 label::ucd (const std::string & text, u_int16 & i) const
 {
-    u_int8 c = my_text_[idx];
+    const u_int8 c = text.at(i);
     if (c < 0x80) return c;
 
     if (c < 0xe0) 
     {
-        u_int8 c1 = my_text_[++idx];
+        const u_int8 c1 = text.at(++i);
         return ((u_int16) (c & 0x1f) << 6)
             |   (u_int16) (c1 ^ 0x80);
     }
     
-    u_int8 c1 = my_text_[++idx];
-    u_int8 c2 = my_text_[++idx];
+    const u_int8 c1 = text.at(++i);
+    const u_int8 c2 = text.at(++i);
     return ((u_int16) (c & 0x0f) << 12)
         |  ((u_int16) (c1 ^ 0x80) << 6)
         |   (u_int16) (c2 ^ 0x80);
 }
-
