@@ -144,14 +144,22 @@ int main(int argc, char * argv[])
         
         str = strrchr (argv[0], '/');
         *(str + 1) = 0;
-        chdir (argv[0]);
+        if (chdir (argv[0]) == -1)
+        {
+        	perror("Failed to set working directory");
+        	return 1;
+        }
 
         *(str + 1) = '/';
     }
     
     // get absolute path to current working directory
     char buf[500];
-    getcwd (buf, sizeof (buf));
+    if (getcwd (buf, sizeof (buf)) == NULL)
+    {
+    	perror("Failed getting absolute path to working directory");
+    	return 1;
+    }
     
     char *str = buf;
     do if (*str == '\\') *str = '/'; 
@@ -213,7 +221,7 @@ int main(int argc, char * argv[])
     {
     	std::cout << "Initializing Python module failed" << std::endl;
         SDL_Quit ();
-    	return false;
+    	return 1;
     }
 
     // init the game data
