@@ -1,15 +1,19 @@
 /*
-   $Id: mapcharacter.cc,v 1.49 2003/02/17 19:31:21 ksterker Exp $
-
    Copyright (C) 1999/2000/2001/2002 Alexandre Courbot
-   Part of the Adonthell Project http://adonthell.linuxgames.com
+   Part of the Adonthell Project <http://adonthell.nongnu.org>
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY.
+   Adonthell is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-   See the COPYING file for more details.
+   Adonthell is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Adonthell.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /** 
@@ -725,7 +729,7 @@ mapcharacter *mapcharacter::whosnext () const
 bool mapcharacter::do_stuff (string method, PyObject *args)
 {
     if (!schedule.has_attribute (method)) return false;
-    else schedule.call_method (method, args);
+    else schedule.call_method (method, args == Py_None ? NULL : args);
     
     return true;
 }
@@ -740,6 +744,7 @@ void mapcharacter::set_schedule (string file, PyObject * args)
     // Set new schedule
     if (file != "")
     {
+    	if (args == Py_None) args = NULL;
         Py_XINCREF (args); 
         schedule_args = args; 
         u_int16 argssize = args == NULL ? 1 : PyTuple_Size (args) + 1; 
