@@ -1,15 +1,19 @@
 /*
-   $Id: win_select.cc,v 1.7 2002/01/05 21:22:56 ksterker Exp $
-   
    (C) Copyright 2000 Joel Vennin
-   Part of the Adonthell Project http://adonthell.linuxgames.com
+   Part of the Adonthell Project <http://adonthell.nongnu.org>
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT AN Y WARRANTY.
+   Adonthell is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-   See the COPYING file for more details
+   Adonthell is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Adonthell.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "win_select.h"
@@ -445,10 +449,10 @@ bool win_select::update()
     {
       if(focus_ && activate_keyboard_ && activated_)
 	{
-	  if(input::has_been_pushed (SDLKey (next_key))) next_();
-	  if(input::has_been_pushed (SDLKey (previous_key))) previous_();
-	  if(input::has_been_pushed (SDLKey (back_key))) back();
-	  if(input::has_been_pushed (SDLKey (activate_key))) activate___();
+	  if(input::has_been_pushed (SDL_Keycode (next_key))) next_();
+	  if(input::has_been_pushed (SDL_Keycode (previous_key))) previous_();
+	  if(input::has_been_pushed (SDL_Keycode (back_key))) back();
+	  if(input::has_been_pushed (SDL_Keycode (activate_key))) activate___();
 	} 
       return true;
     }
